#!/bin/bash
# ISDN-TOOL - Easy dialog-based configuration of ISDN devices
# (C) Klaus Knopper Aug 2004

PATH="/bin:/sbin:/usr/bin:/usr/sbin:/usr/X11R6/bin:/usr/local/bin"
export PATH

# Get root
[ "`id -u`" != "0" ] && exec sudo "$0" "$@"
unset SUDO_COMMAND

XDIALOG_HIGH_DIALOG_COMPAT=1
export XDIALOG_HIGH_DIALOG_COMPAT

XDIALOG_FORCE_AUTOSIZE=1
export XDIALOG_FORCE_AUTOSIZE

XTERM=rxvt
type -a rxvt >/dev/null 2>&1 || XTERM=xterm

TMP="/tmp/isdntool.$$"

bailout(){
rm -f "$TMP"
exit $1
}

DIALOG="dialog"
[ -n "$DISPLAY" ] && [ -x /usr/bin/Xdialog ] && DIALOG="Xdialog"

trap bailout 1 2 3 15

BACKTITLE="KNOPPIX ISDN-TOOL"

[ -f /etc/sysconfig/knoppix ] && . /etc/sysconfig/knoppix
[ -z "$LANG" ] && export LANG
[ -z "$LANGUAGE" ] && export LANGUAGE
[ -z "$CHARSET" ] && export CHARSET

# Language-dependent Messages
case "$LANGUAGE" in
de*|at*|ch*)
NO_ISDNCARD="Standard"
EXIT="Beenden"
STATUS="Verbinden"
CURRENTPROVIDER="Keiner ausgewhlt"
MAIN="Hauptmen"
MSN="Eigene Telefonnummer"
ENTER_MSN="Bitte eigene Telefonnummer OHNE VORWAHL fr diesen Anschlus eingeben. Bei Eingabe von * wird automatisch eine an diesem Anschluss verfgbare Anschlusskennung ausgewhlt."
PPHONE="Provider Telefonnummer"
ENTER_PPHONE="Bitte geben Sie die VOLLSTNDIGE Telefonnummer Ihres Internet-Anbieters ein:"
PLOGIN="Provider Login"
PPASSWORD="Provider Passwort"
SETTINGS="ISDN-Einstellungen"
EDIT="Anschauen/Bearbeiten"
PROVIDER="Provider"
HARDWARE="Hardware"
HWMENU="Die ISDN-Hardware wird mit der folgenden Anweisung initialisiert, wenn Sie \"OK\" whlen. Bitte korrigieren Sie ggf. die hardwarespezifischen Parameter wie IO-Adresse und Interrupt, falls erforderlich."
TITLE_GAUGE="Arbeite..."
INITHW="Hardware-Initialisierung..."
SAVE="Einstellungen speichern/aktivieren"
INVISIBLE="unsichtbar"
NODRIVER="offline, kein Treiber geladen"
SHOW_STATUS="Online-Status anzeigen"
START="ISDN-System starten"
STOP="ISDN-System anhalten"
BACK="Zurck"
DIALHELP="Das ISDN-Subsystem ist nun vollstndig konfiguriert. Damit automatisch beim Aufruf eines Internet-Dienstes eine Verbindung aufgebaut wird, whlen Sie bitte im Hauptmen \"$STATUS\" und dort \"$START\". Wenn Sie das ISDN-Subsystem lieber spter manuell starten mchten, geben Sie in der Shell bitte \"/etc/init.d/isdnutils start\" ein. Das Kommando \"/etc/init.d/isdnutils stop\" trennt alle ISDN-Verbindungen."
CONFIGHELP="Das ISDN-Subsystem ist noch nicht initialisiert. Damit eine Internet-Einwahl per ISDN mglich ist, muss das richtige Hardware-Modul und die Provider-Informationen korrekt angegeben werden."
;;
*)
NO_ISDNCARD="Default"
STATUS="Connect"
CURRENTPROVIDER="None"
MAIN="Main Menu"
MSN="My own phone number"
ENTER_MSN="Please enter your own phone number for this connection, WITHOUT PREFIXES OR AREA CODE. Entering * will use any available phone number on his line automatically."
PPHONE="Provider phone number"
ENTER_PPHONE="Please enter the FULL PHONE NUMBER of your internet provider:"
PLOGIN="Provider Login"
PPASSWORD="Provider Password"
SETTINGS="ISDN-Settings"
EDIT="View/Edit"
PROVIDER="Provider"
HARDWARE="Hardware"
TITLE_GAUGE="Working..."
HWMENU="The ISDN hardware will be configured using the following command if you hit \"OK\". Please correct the hadrware specific parameters like io-address and interrupt, if necessary."
INITHW="Initializing hardware..."
SAVE="Save/Activate Settings"
INVISIBLE="not shown"
NODRIVER="offline, no driver loaded"
SHOW_STATUS="show online/offline status"
START="Start ISDN System"
STOP="Stop ISDN System"
BACK="Return"
EXIT="Exit"
DIALHELP="The ISDN subsystem is now fully configured. To start an ISDN connection automatically when accessing internet services, please select \"$STATUS\" from the main menu, and \"$START\". If you'd rather like to start the ISDN service manually, enter \"/etc/init.d/isdnutils start\" in the shell. To disconnect all ISDN connections, enter the command \"/etc/init.d/isdnutils stop\"."
CONFIGHELP="The ISDN subsystem is not initialised yet. In order to create an internet connection via ISDN, you need to specify the hadrware module matching your card, and insert your provider's data correctly."
;;
esac

# Default Settings
CURRENTSTATE="Offline"
ISDN_MSN="*"
ISDN_PROVIDER_PHONE=""
ISDN_PROVIDER_LOGIN=""
ISDN_PROVIDER_PASSWORD=""
ISDNCARD_NAME="$NO_ISDNCARD"
ISDNCARD_COMMAND=""

# Read in previously saved values
[ -f /etc/sysconfig/isdn ] && . /etc/sysconfig/isdn

gauge(){
rm -f "$TMP.done"
status=0
while [ ! -e "$TMP.done" ]; do echo "$status" ; status="`expr \( 100 - $status \) / 4 + $status`"; sleep 2; done | $DIALOG --title "$TITLE_GAUGE" --gauge "$1" 8 75 0
}

# Stop status bar
killgauge(){
touch "$TMP.done" ; wait ; rm -f "$TMP.done"
}

# writeconfig variable_name variable_content
writeconfig(){
rm -f "$TMP"
egrep -v "(^$1)" /etc/sysconfig/isdn >"$TMP" 2>/dev/null
echo "$1=\"$2\"" >> "$TMP"
cat "$TMP" > /etc/sysconfig/isdn
rm -f "$TMP"
}
 
hardware(){
# Name "Command to initialize driver".
CARDS=("Auto-Detect" "" \
"ACER P10" "modprobe hisax type=30 irq=5 io=0x300" \
"ASUS COM ISDNLink" "modprobe hisax type=12 irq=5 io=0x200" \
"AVM A1 (Fritz)" "modprobe hisax type=5 irq=10 io=0x300" \
"AVM B1 PCI Adapter" "modprobe b1pci" \
"AVM Fritz!Bluetooth" "ciptool search; modprobe capi; modprobe capidrv" \
"AVM Fritz!USB" "modprobe fcusb2; modprobe capi; modprobe capidrv; avmcapictrl load fus2base.frm" \
"AVM Fritz!PCI V1.0/V2.0" "modprobe hisax_fcpcipnp" \
"AVM PnP", "modprobe hisax type=27 irq=5 io=0x300" \
"AVM T1 ISDN" "modprobe t1pci" \
"Compaq ISDN S0 ISA" "modprobe hisax type=19 irq=5 io=0x0000 io1=0x0000 io2=0x0000" \
"Creatix Teles PnP" "modprobe hisax type=4 irq=5 io=0x0000 io1=0x0000" \
"Dr. Neuhaus (PnP) Niccy" "modprobe hisax type=24 irq=5 io1=0x0000 io2=0x0000" \
"Dr. Neuhaus (PCI) Niccy" "modprobe hisax type=24 pci_id=12671016" \
"E.D. Diva ISA PnP" "modprobe hisax type=11 irq=9 io=0x180" \
"E.D. Diva 20PRO PCI" "modprobe hisax type=11 pci_id=1133e001" \
"E.D. Diva 20 PCI" "modprobe hisax type=11 pci_id=1133e002" \
"E.D. Diva 20PRO_U PCI" "modprobe hisax type=11 pci_id=1133e003" \
"E.D. Diva 20_U PCI" "modprobe hisax type=11 pci_id=1133e004" \
"ELSA PCC/PCF" "modprobe hisax type=6" \
"ELSA Quickstep 1000" "modprobe hisax type=7 irq=5 io=0x300" \
"ELSA Quickstep 1000 PCI" "modprobe hisax type=18 pci_id=10481000" \
"ELSA Quickstep 3000 PCI" "modprobe hisax type=18 pci_id=10483000" \
"Gazel cards ISA" "modprobe hisax type=34 irq=5 io=0x300" \
"Gazel cards PCI" "modprobe hisax type=34 pci_id=10b51030" \
"HFC-2BS0 based ISA" "modprobe hisax type=13 irq=9 io=0xd80" \
"HFC-2BS0 based PCI" "modprobe hisax type=35 pci_id=13972bd0" \
"HST Saphir" "modprobe hisax type=31 irq=5 io=0x300" \
"ITK ix1-micro Rev.2" "modprobe hisax type=9 irq=9 io=0xd80" \
"MIC card" "modprobe hisax type=17 irq=9 io=0xd80" \
"NETjet PCI" "modprobe hisax type=20 pci_id=e1590001" \
"Sedlbauer PC 104" "modprobe hisax type=15 irq=9 io=0xd80" \
"Sedlbauer Speed PCI" "modprobe hisax type=15" \
"Sedlbauer Speed Card" "modprobe hisax type=15 irq=9 io=0xd80" \
"Sedlbauer Speed Fax+" "modprobe hisax type=28 irq=9 io=0xd80; hisaxctrl HiSax 9 /usr/share/isdn/ISAR.BIN" \
"Sedlbauer Speed fax+ PCI" "modprobe hisax type=28 pci_id=e1590002; hisaxctrl HiSax 9 /usr/share/isdn/ISAR.BIN" \
"Siemens I-Surf 1.0" "modprobe hisax type=29 irq=9 io=0xd80 mem=0xd000" \
"Telekom A4T" "modprobe hisax type=32" \
"Teles 8.0" "modprobe hisax type=2 irq=9 mem=0xd800" \
"Teles 16.0" "modprobe hisax type=1 irq=5 io=0xd80 mem=0xd000" \
"Teles 16.3" "modprobe hisax type=3 irq=9 io=0xd80" \
"Teles 16.3c PnP" "modprobe hisax type=14 irq=9 io=0xd80" \
"Teles PCI" "modprobe hisax type=21" \
"Teles PnP" "modprobe hisax type=4 irq=5 io=0x0000 io1=0x0000" \
"Teles S0Box" "modprobe hisax type=25 irq=7 io=0x378" \
"USR Sportster intern" "modprobe hisax type=16 irq=9 io=0xd80" \
"W6692 based PCI cards" "modprobe hisax type=36 pci_id=10506692")
# Rausgenommen, da durch hisax_fcpcipnp ersetzt: "AVM Fritz!PCI" "modprobe hisax type=27 pci_id=12440a00"
rm -f "$TMP"
$DIALOG --cancel-label "$BACK" --backtitle "$BACKTITLE" --title "$HARDWARE" --menu "" 15 75 8 "$ISDNCARD_NAME" "$ISDNCARD_COMMAND" "${CARDS[@]}" 2>"$TMP" || { rm -f "$TMP"; return 0; }
read ISDNCARD_NAME <"$TMP" ; rm -f "$TMP"
for((i=0;i<${#CARDS[@]};i+=2)); do
[ "$ISDNCARD_NAME" = "${CARDS[$i]}" ] && i=$(($i + 1)) && ISDNCARD_COMMAND="${CARDS[$i]}" && break
done
if [ -n "$ISDNCARD_NAME" -a -n "$ISDNCARD_COMMAND" ]; then
$DIALOG --backtitle "$BACKTITLE" --title "$HARDWARE" --inputbox "$HWMENU" 12 75 "$ISDNCARD_COMMAND" 2>"$TMP" || { rm -f "$TMP"; return 0; }
read ISDNCARD_COMMAND < "$TMP"
fi
if [ -n "$ISDNCARD_NAME" ]; then
inithw
fi
}

settings(){
while true; do
SETTINGS_MENU=("$HARDWARE" "$ISDNCARD_NAME" \
"$MSN" "$ISDN_MSN" \
"$PPHONE" "$ISDN_PROVIDER_PHONE" \
"$PLOGIN" "$ISDN_PROVIDER_LOGIN" \
"$PPASSWORD" "*** $INVISIBLE ***" \
"$SAVE" "")
rm -f "$TMP"
$DIALOG --cancel-label "$BACK" --backtitle "$BACKTITLE" --title "$SETTINGS" --menu "" 15 75 8 "${SETTINGS_MENU[@]}" 2>"$TMP" || break
case "$(<$TMP)" in
$MSN) msn;;
$PPHONE) pphone;;
$PLOGIN) plogin;;
$PPASSWORD) ppassword;;
$HARDWARE) hardware;;
$SAVE) savesettings; break;;
esac
done
rm -f "$TMP"
if [ -n "$ISDNCARD_NAME" -a -n "$ISDN_MSN" -a -n "$ISDN_PROVIDER_PHONE" -a -n "$ISDN_PROVIDER_LOGIN" -a -n "$ISDN_PROVIDER_PASSWORD" ] && isdnstatus >/dev/null 2>&1; then
$DIALOG --backtitle "$BACKTITLE" --title "$SETTINGS" --msgbox "$DIALHELP" 12 70
return 0
else
$DIALOG --backtitle "$BACKTITLE" --title "$SETTINGS" --msgbox "$CONFIGHELP" 11 65
return 1
fi
}

msn(){
rm -f "$TMP"
$DIALOG --backtitle "$BACKTITLE" --title "$MSN" --inputbox "$ENTER_MSN" 13 55 "$ISDN_MSN" 2>"$TMP" || { rm -f "$TMP"; return 0; }
perl -pi -e "s/[^0-9\*]//g" "$TMP"
ISDN_MSN="$(<$TMP)"
rm -f "$TMP"
}

pphone(){
rm -f "$TMP"
$DIALOG --backtitle "$BACKTITLE" --title "$PPHONE" --inputbox "$ENTER_PPHONE" 12 55 "$ISDN_PROVIDER_PHONE" 2>"$TMP" || { rm -f "$TMP"; return 0; }
perl -pi -e "s/[^0-9]//g" "$TMP"
ISDN_PROVIDER_PHONE="$(<$TMP)"
rm -f "$TMP"
}

plogin(){
rm -f "$TMP"
$DIALOG --backtitle "$BACKTITLE" --title "$PLOGIN" --inputbox "" 8 55 "$ISDN_PROVIDER_LOGIN" 2>"$TMP" || { rm -f "$TMP"; return 0; }
ISDN_PROVIDER_LOGIN="$(<$TMP)"
rm -f "$TMP"
}

ppassword(){
rm -f "$TMP"
if [ "$DIALOG" = "Xdialog" ]; then
$DIALOG --password --backtitle "$BACKTITLE" --title "$PPASSWORD" --inputbox "" 8 55 "$ISDN_PROVIDER_PASSWORD" 2>"$TMP" || { rm -f "$TMP"; return 0; }
else
$DIALOG --insecure --backtitle "$BACKTITLE" --title "$PPASSWORD" --passwordbox "" 8 55 "$ISDN_PROVIDER_PASSWORD" 2>"$TMP" || { rm -f "$TMP"; return 0; }
fi
ISDN_PROVIDER_PASSWORD="$(<$TMP)"
rm -f "$TMP"
}

addprovider(){
perl -pi -e "s/^REMOTEMSN=.*\$/REMOTEMSN=\"$ISDN_PROVIDER_PHONE\"/g;s/^LOCALMSN=.*\$/LOCALMSN=\"$ISDN_MSN\"/g;" /etc/isdn/device.ippp0{,+1} 
perl -pi -e "s/^name .*$/name ${ISDN_PROVIDER_LOGIN/@/\\@}/g;s/^remotename .*\$/remotename $ISDN_PROVIDER_PHONE/g" /etc/isdn/ipppd.ippp0{,+1} 
for i in pap chap; do
 rm -f "$TMP"
 egrep -v "\"$ISDN_PROVIDER_LOGIN\" \"$ISDN_PROVIDER_PHONE\"" /etc/ppp/$i-secrets >"$TMP" 2>/dev/null
 cat "$TMP" >/etc/ppp/$i-secrets
 echo "\"$ISDN_PROVIDER_LOGIN\" \"$ISDN_PROVIDER_PHONE\" \"$ISDN_PROVIDER_PASSWORD\"" >>/etc/ppp/$i-secrets
done
}

savesettings(){
writeconfig ISDNCARD_NAME "$ISDNCARD_NAME"
writeconfig ISDNCARD_COMMAND "$ISDNCARD_COMMAND"
writeconfig ISDN_MSN "$ISDN_MSN"
writeconfig ISDN_PROVIDER_PHONE "$ISDN_PROVIDER_PHONE"
writeconfig ISDN_PROVIDER_LOGIN "$ISDN_PROVIDER_LOGIN"
writeconfig ISDN_PROVIDER_PASSWORD "$ISDN_PROVIDER_PASSWORD"
addprovider
}

isdnstatus(){
read CURRENTSTATE <<EOT
$(isdnctrl addif ippp0 2>/dev/null; isdnctrl status ippp0 2>/dev/null)
EOT
if [ -n "$CURRENTSTATE" ]; then
RC=0
else
CURRENTSTATE="$NODRIVER"
RC=1
fi
echo "$CURRENTSTATE"
return $RC
}

status(){
ISDNSTATUS="$(isdnstatus)"
STATUS_MENU=("$SHOW_STATUS" "(${ISDNSTATUS})" \
"$START" "" \
"$STOP" "")
while true; do 
rm -f "$TMP"
$DIALOG --cancel-label "$BACK" --backtitle "$BACKTITLE" --title "$STATUS" --menu "" 11 75 4 "${STATUS_MENU[@]}" 2>"$TMP" || break
case "$(<$TMP)" in
$SHOW_STATUS) show_status;;
$START) start;;
$STOP) stop;;
esac
done
rm -f "$TMP"
return 0
}

show_status(){
[ -n "$DISPLAY" ] && $XTERM -e imon || imon
}

inithw(){
gauge "$INITHW" &
sleep 2
# Nothing works of hisax is loaded with wrong parameters.
for i in hisax_isac hisax_st5481 hisax_fcpcipnp hisax; do rmmod "$i" 2>/dev/null; done

case "$ISDNCARD_COMMAND" in
 *hisax*) ;;
 *) # Try to detect fritzcard
    isdnstatus >/dev/null 2>&1 || { [ -x /usr/sbin/detect-fc ] && /usr/sbin/detect-fc >/dev/null 2>&1 ; modprobe capidrv; modprobe capi; } ;;
esac

# Create missing files
[ -e /dev/MAKEDEV ] || ln -s /sbin/MAKEDEV /dev/MAKEDEV 2>/dev/null
[ -e /dev/isdnctrl0 ] || mknod -m 666 /dev/isdnctrl0 c 45 64 2>/dev/null
[ -e /dev/isdnctrl ] || ln -s isdnctrl0 /dev/isdnctrl 2>/dev/null
[ -e /dev/isdninfo ] || mknod -m 444 /dev/isdninfo c 45 255 2>/dev/null
[ -n "$ISDNCARD_COMMAND" ] && $ISDNCARD_COMMAND
killgauge
}

start(){
inithw
grep ^nameserver /etc/resolv.conf 2>/dev/null || echo "nameserver 10.0.0.2" >/etc/resolv.conf
/etc/init.d/isdnutils restart
}

stop(){
/etc/init.d/isdnutils stop
RESOLVCONF="$(grep -v '^nameserver 10.0.0.2' /etc/resolv.conf 2>/dev/null)"
[ "$?" = "0" ] && echo "$RESOLVCONF" > /etc/resolv.conf
}

main(){
[ -f /etc/sysconfig/isdn ] && . /etc/sysconfig/isdn
CURRENTSTATE="$(isdnstatus)"
rm -f "$TMP"
$DIALOG --cancel-label "$EXIT" --backtitle "$BACKTITLE" --title "$MAIN" --menu "" 12 75 5 "${STATUS}:" "${CURRENTSTATE}" "${SETTINGS}" "$EDIT" 2>"$TMP"
[ "$?" != "0" ] && return 1
read SELECTION <"$TMP"
rm -f "$TMP"
case "$SELECTION" in
 $STATUS*)   status;;
 $SETTINGS*) settings;;
 *)          return 5;;
esac
return 0
}

# MAIN
while true; do main || break; done
bailout 0
